#include "include/wl_def.h"
#include "include/32x.h"
#include "include/debug_32x.h"


extern void *audiohed;
extern void *audiot;
extern void *gamemaps;
extern void *maphead;
extern void *vgahead;
extern void *vgagraph;
extern void *vswap;


/*
=============================================================================

						 GLOBAL VARIABLES

=============================================================================
*/

int mapon;

word	*mapsegs[MAPPLANES];
maptype	*mapheaderseg[NUMMAPS];
byte	*audiosegs[NUMSNDCHUNKS];
byte	*grsegs[NUMCHUNKS];

static word	maps[MAPPLANES][64*64];

static long *grstarts = &vgahead;	/* array of offsets in vgagraph */
static long *audiostarts = &audiohed; /* array of offsets in audiot */

/*
=============================================================================

					 CACHE MANAGER ROUTINES

=============================================================================
*/

/*
======================
=
= CAL_SetupGrFile
=
======================
*/

static void CAL_SetupGrFile()
{
	byte *grtemp;
	int i;

    /* load the pic headers into pictable */
	grtemp = (byte *)((int)&vgagraph + grstarts[STRUCTPIC]);
	for (i = 0; i < NUMPICS; i++) {
		pictable[i].width = grtemp[i*4+0] | (grtemp[i*4+1] << 8);
		pictable[i].height = grtemp[i*4+2] | (grtemp[i*4+3] << 8);
	}
}

/* ======================================================================== */

/*
======================
=
= CAL_SetupMapFile
=
======================
*/

static void CAL_SetupMapFile()
{
	int i;
	long *mhead;
	long pos;

    /* load all map headers */
    mhead = &maphead;

	for (i = 0; i < NUMMAPS; i++)
	{
		pos = *mhead++;
		if (pos == 0)
		    mapheaderseg[i] = (maptype *)NULL;
        else
            mapheaderseg[i] = (maptype *)((int)&gamemaps + pos);
	}

	for (i = 0; i < MAPPLANES; i++)
		mapsegs[i] = &maps[i][0];
}


/* ======================================================================== */

/*
======================
=
= CAL_SetupAudioFile
=
======================
*/

static void CAL_SetupAudioFile()
{
}

/* ======================================================================== */

/*
======================
=
= CA_Startup
=
= Open all files and load in headers
=
======================
*/

void CA_Startup()
{
	CAL_SetupMapFile();
	CAL_SetupGrFile();
	CAL_SetupAudioFile();

	mapon = -1;
}

/*
======================
=
= CA_Shutdown
=
= Closes all files
=
======================
*/

void CA_Shutdown()
{
}

/* ======================================================================== */

/*
======================
=
= CA_CacheAudioChunk
=
======================
*/

void CA_CacheAudioChunk(int chunk)
{
	int pos = audiostarts[chunk];

	audiosegs[chunk] = (byte *)((int)&audiot + pos);
}

void CA_UnCacheAudioChunk(int chunk)
{
}

/*
======================
=
= CA_LoadAllSounds
=
======================
*/

void CA_LoadAllSounds()
{
	int start, i;

	for (start = STARTADLIBSOUNDS, i = 0; i < NUMSOUNDS; i++, start++)
		CA_CacheAudioChunk(start);
}

/* ======================================================================== */

/*
======================
=
= CA_CacheGrChunk
=
= Makes sure a given chunk is in memory, loadiing it if needed
=
======================
*/

void CA_CacheGrChunk(int chunk)
{
	long pos = grstarts[chunk];

	grsegs[chunk] = (byte *)((int)&vgagraph + pos);
}

void CA_UnCacheGrChunk(int chunk)
{
}

/* ======================================================================== */

/*
======================
=
= CA_CacheMap
=
======================
*/

void CA_CacheMap(int mapnum)
{
	long pos;
	int plane;

	mapon = mapnum;

    /* load the planes into the buffers */
	for (plane = 0; plane < MAPPLANES; plane++)
	{
		pos = mapheaderseg[mapnum]->planestart[plane];
		memcpy((void *)&maps[plane][0], (void *)((int)&gamemaps + pos), 64*64*2);
	}
}

/* ======================================================================== */

static boolean PMStarted;

int ChunksInFile, PMSpriteStart, PMSoundStart;

memptr PMPages[MAX_CHUNKS];

static void PML_OpenPageFile()
{
	int i, off;
	int *iptr = (int *)&vswap;

	/* get header variables */
	ChunksInFile = *iptr++;
	PMSpriteStart = *iptr++;
	PMSoundStart = *iptr++;

	/* get chunk offsets, make ptrs */
	for (i = 0; i < ChunksInFile; i++) {
	    off = *iptr++;
		PMPages[i] = off ? (memptr)((int)&vswap + off) : (memptr)0;
		//PMPages[i] = (memptr)((int)&vswap + off);
    }
}

static void PML_ClosePageFile()
{
}

memptr PM_GetPage(int pagenum)
{
	if (pagenum >= ChunksInFile) {
	    char temp[40];
	    sprintf(temp, "Pagenum %d\n", pagenum);
	    Debug32xScreenPuts(temp);
		Quit("PM_GetPage: Invalid page request");
    }

	if (PMPages[pagenum] == 0) {
	    char temp[40];
	    sprintf(temp, "Pagenum %d\n", pagenum);
	    Debug32xScreenPuts(temp);
		Quit("PM_GetPage: No page");
    }

	return PMPages[pagenum];
}

void PM_Startup()
{
	if (PMStarted)
		return;

	PML_OpenPageFile();

	PMStarted = true;
}

void PM_Shutdown()
{
	if (!PMStarted)
		return;

	PML_ClosePageFile();

	PMStarted = false;
}
